<?php

namespace Drupal\ocr_image\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\file\Entity\File;
use Drupal\file\Plugin\Field\FieldWidget\FileWidget;
use Drupal\ocr_image\Plugin\Field\TesseractLanguages;

/**
 * Plugin implementation of the 'Ocr pdf' widget.
 *
 * @FieldWidget(
 *   id = "ocr_pdf",
 *   label = @Translation("Ocr/parser file"),
 *   field_types = {
 *     "file"
 *   }
 * )
 */
class OcrFileWidget extends FileWidget {

  use Base;

  /**
   * {@inheritDoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $element = parent::formElement($items, $delta, $element, $form, $form_state);
    $field_name = $items->getName();
    if (!empty($field_text_name = $this->getSetting('text_field'))) {
      $fid = $form_state->getValue($field_name);
      if (!empty($fid)) {
        if (is_array($fid)) {
          $fid = end($fid);
          if (is_array($fid) && !empty($fid["fids"])) {
            $fid = end($fid["fids"]);
          }
        }
        $file = File::load($fid);
        $uri = $file->getFileUri();
        $stream_wrapper_manager = \Drupal::service('stream_wrapper_manager')
          ->getViaUri($uri);
        $file_path = $stream_wrapper_manager->realpath();
        $language = $this->getSetting('language');
        if (empty($language)) {
          $language = TesseractLanguages::tesseractConvertLang();
        }
        $doc_parser_service = \Drupal::service('ocr_image.DocParser');
        $line_datas = $doc_parser_service->getText($file_path, $language, $this->getSetting('limit'));
        if (!empty($line_datas)) {
          $ocr = implode(" " . PHP_EOL, $line_datas);
          $form_state->setValue($field_text_name, $ocr);
          $element["#default_value"]["description"] = strip_tags($ocr);
          $element['#attached']['drupalSettings']['ocr_field_widget'] = [
            'field_name' => $field_name,
            'field' => $field_text_name,
            'ocr' => $ocr,
          ];
          $element['#attached']['library'][] = 'ocr_image/ocr_image';
        }
      }
    }

    return $element;
  }

}
