<?php

namespace Drupal\ocr_image\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\file\Entity\File;
use Drupal\image\Plugin\Field\FieldWidget\ImageWidget;

/**
 * Plugin implementation of the 'Ocr image' widget.
 *
 * @FieldWidget(
 *   id = "ocr_image",
 *   label = @Translation("Ocr image"),
 *   field_types = {
 *     "image"
 *   }
 * )
 */
class OcrImageWidget extends ImageWidget {

  use Base;

  /**
   * {@inheritDoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $element = parent::formElement($items, $delta, $element, $form, $form_state);
    $field_name = $items->getName();
    if (!empty($field_text_name = $this->getSetting('text_field'))) {
      $language = $this->getSetting('language');
      $fid = $form_state->getValue($field_name);
      if (!empty($fid)) {
        if (is_array($fid)) {
          $fid = end($fid);
          if (is_array($fid) && !empty($fid["fids"])) {
            $fid = end($fid["fids"]);
          }
        }
        $file = File::load($fid);
        $uri = $file->getFileUri();
        $stream_wrapper_manager = \Drupal::service('stream_wrapper_manager')
          ->getViaUri($uri);
        $file_path = $stream_wrapper_manager->realpath();
        $ocr_image_service = \Drupal::service('ocr_image.OcrImage');
        $image_text = $ocr_image_service->getText($file_path, $language, $this->getSetting('limit'));
        $full_text = $image_text['full_text'];
        if (!empty($full_text)) {
          $form_state->setValue($field_text_name, $full_text);
          $element["#default_value"]["alt"] = $image_text['alt'];
          $element["#default_value"]["title"] = $image_text['title'];
          $element['#attached']['drupalSettings']['ocr_field_widget'] = [
            'field_name' => $field_name,
            'field' => $field_text_name,
            'ocr' => $full_text,
          ];
          $element['#attached']['library'][] = 'ocr_image/ocr_image';
        }
      }
    }

    return $element;
  }

}
